#ifndef __ALL_CYCLES
#define __ALL_CYCLES

#include "command/one_cycle.js"
#include "cycle/cycle.js"
#include "max/enable.js"
#include "command/vl_cycles.js"
#include "command/vl_program_changes.js"

function AllCycles(context, vspacing, edit, cycle, depth, pc) {
  this.objects = [];
  this.cycle = cycle || VL.ReadCycleDefinition();
  this.pc = pc || VL.program_changes;
  this.edit = edit || 'locked';
  this.vspacing = vspacing;
  this.SetDepth(depth || 3);
};

AllCycles.prototype = {
  get length() { return this.objects.length; }
};

AllCycles.prototype.UpdateUI = function() {
  var is_edit = (this.edit == 'edit');
  for (var i = 0; i < this.objects.length; ++i)
    this.objects[i].UpdateUI(this.cycle.Get(i + 1), is_edit);
};

AllCycles.prototype.SetDepth = function(depth) {
  if (depth == this.objects.length)
    return;

  if (depth < this.objects.length) {
    for (var i = depth; i < this.objects.length; ++i)
      this.objects[i].Remove();
    this.objects = this.objects.slice(0, depth);

  } else {
    for (var i = this.objects.length; i < depth; ++i) {
      var one_row = new Commander.OneCycle(i, this.vspacing);
      one_row.CreateAndConnect(context);
      this.objects.push(one_row);
    }
  }
  this.UpdateUI();
};

AllCycles.prototype.Advance = function(level, delta) {
  if (delta === undefined)
    delta = 1;

  if (this.cycle.Get(level).Advance(delta)) {
    this.UpdateUI();
    var pc = this.cycle.Get(3);
    if (!(pc in this.pc)) {
      post("Didn't find pc", pc, '\n');
    } else {
      outlet(0, 'bank', this.pc[pc][0]);
      outlet(0, 'pc', this.pc[pc][1]);
    }
  }
};

AllCycles.prototype.SetName = function(level, name) {
  this.cycle.Get(level).name = name;
};

AllCycles.prototype.Insert = function(level) {
  this.cycle.Get(level).Insert(Cycle.Encycle(this.length - level));
  this.UpdateUI();
};

AllCycles.prototype.Remove = function(level) {
  this.cycle.Remove(level);
};

#endif  // __ALL_CYCLES
