#ifndef __LEVEL_MANAGER
#define __LEVEL_MANAGER

#include "cycle/named_cycle.js"
#include "util/dict_utils.js"

function LevelManager(depth) {
  var top;
  var level_to_name_map = [];
  var me = this;

  function GetNameMap(level) {
    return DictUtils.GetOrAddDefault(level_to_name_map, level, {});
  };

  function Execute() {
    var cmds = me.Get(0);
    for (var i = 0; i < cmds.length; ++i)
      cmds[i]();

    return true;
  };

  this.Assign = function(level, item) {
    top = item;
    for (; level < depth; ++level)
      top = new Cycle([top]);

    return Execute();
  };

  this.Create =  function(level, name, desc) {
    var item = level ? NamedCycle(desc, level - 1) : desc;
    GetNameMap(level)[name] = item;
    me.Assign(level, item);
    return true;
  };

  this.Advance = function(level, delta) {
    return this.Get(level).Advance(delta) && Execute();
  };

  this.Get = function(level) {
    level = level | 0;
    var item = top;
    for (var i = depth; i > level; --i) {
      if (item.list.length == 0 && item.name)
        item = level_to_name_map[i][item.name];
      item = item.Get();
    };
    return item;
  };
};

#endif  // __LEVEL_MANAGER
