#ifndef __MAX_CALLBACK
#define __MAX_CALLBACK

#include "max/outlet.js"
#include "util/array.js"
#include "util/functional.js"

var Callback = new Object();

// Callback.Add() adds a callback to a source object's member, so that when
// source[name] is changed, cb
Callback.Add = function(source, name, cb) {
  // To avoid an infinite loop, we store the value under a new name.
  var n = '_' + name;
  source.__defineGetter__(name, function()  { return source[n]; });
  source.__defineSetter__(name, function(x) { source[n] = x; cb(name, x); });
};

// Callback.AddOutletFiltered adds a callback that sends filtered data to an
// outlet number.
//
// For convenience, method_names can be either an array or an associative array:
// see Array.ForEach for details.
Callback.AddOutletFiltered = function(source, members, out, filter) {
  function AddOutletCallback(name, maxname) {
    Callback.Add(source, name, function(_, value) {
      _outlet(out | 0, filter(maxname, value));
    });
  };
  Array.ForEach(members, AddOutletCallback);
};

Callback.AddOutlet = function(source, members, out) {
  function ConcatFilter(maxname, value) {
    return [maxname].concat(value);
  };

  Callback.AddOutletFiltered(source, members, out, ConcatFilter);
};

Callback.AddTexteditOutlet = function(source, members, out) {
  function TexteditFilter(maxname, value) {
    value = (value + '').replace(/^\s+|\s+$/g, '');  // Remove whitespace.
    // TODO(tritchford):  what about ","?
    return value.length ? [maxname, 'set', value] : [maxname, 'clear'];
  };

  return Callback.AddOutletFiltered(source, members, out, TexteditFilter);
};

// Callback.AddInlet adds callbacks to the object "jsthis" that set a member or
// call a method on the object "target".
//
// Usually "jsthis" is your "top-level this" so incoming Max messages are
// automatically translated into method calls or setters on "target".  Neat!
Callback.AddInlet = function(jsthis, target, members) {
  function AddInlet(name, max) {
    jsthis[max.toLowerCase()] = Functional.ObjectApplier(target, name);
  };
  Array.ForEach(members, AddInlet);
};

#endif  // __MAX_CALLBACK
