#ifndef __CREATE_MIDIINOUT
#define __CREATE_MIDIINOUT

#include "util/dict_utils.js"

function CreateMidiInOut(desc, js) {
  function create(left, top, name, rest) {
    var object = Max.patcher.newdefault(left, top, name, rest);
    var words = [name].concat(rest || []);
    var right = left;

    for (var i in words)
      right += 6 * words[i].length + (i ? 6 : 0);

    object.rect = [left, top, right, object.rect[3]];
    return object;
  };

  var max_objects = {};
  var types = DictUtils.Keys(desc);

  var top_y = 20;
  var route_y = top_y + 60;
  var subroute_y = route_y + 32;
  var child_y = subroute_y + 32;

  var left_x = 8;
  var child_delta_x = 6;

  var route = create(left_x, route_y, 'route', types);
  Max.patcher.connect(js, 0, route, 0);

  var x = left_x;
  for (var i in types) {
    var type = types[i];
    var names = DictUtils.Keys(desc[type]);
    var subroute = create(x, subroute_y, 'route', names);
    Max.patcher.connect(route, i, subroute, 0);
    max_objects[type] = {};

    for (var j in names) {
      var name = names[j];
      var object = create(x, child_y, type, name);
      max_objects[type][name] = object;
      x = object.rect[2] + child_delta_x;
      Max.patcher.connect(subroute, j | 0, object, 0);

      if (type == 'tmidiin')
        Max.patcher.connect(object, 1, js, 0);
    }
  };
  return max_objects;
};

#endif  // __CREATE_MIDIINOUT
