#ifndef __MORSE_CALLBACKS
#define __MORSE_CALLBACKS

#include "max/callback.js"
#include "morse/morse.js"

// Morse.AddCallback() adds callback to and from the various components of a
// Morse.Generator into jsthis, the Max javascript environment.
Morse.AddCallbacks = function(generator, jsthis, outlet_number) {
  if (!generator.callback_added) {
    generator.callback_added = true;

    // Inlet callback automatically route Max messages to setters or methods.

    // Route incoming Max messages to setters and methods on generator.
    Callback.AddInlet(jsthis, generator,
                      ['midioffset', 'pitch', 'looping', 'velocity',
                       'Clear', 'Init', 'KeyIn', 'Mode', 'NoteIn']);

    // Now an incoming Max message like [pitch 1] will set the value of
    // generator.pitch to be 1, or an incoming Max message like [init] will
    // call generator.Init().

    // Route to methods on contained objects...
    Callback.AddInlet(jsthis, generator.tasker, ['Run', 'Stop']);

    // ...or to setters on contained objects, or unrelated objects even.
    Callback.AddInlet(jsthis, generator.model,
                      ['dash', 'dot', 'gap', 'sep', 'space', 'wpm']);

    // Outlet callback are attached to class member variables - setting that
    // member causes messages to be sent to a Max outlet.
    Callback.AddTexteditOutlet(generator.message, {'display': 'message'},
                               outlet_number);

    // The variable generator.makenote is directly attached to the outlet
    // listener with no filter at all - this means that you execute
    //
    //   generator.makenote = [64, 100]
    //
    // and [makenote 64 100] is emitted from the outlet.
    Callback.AddOutlet(generator, ['makenote'], outlet_number);

    // These are attached with a filter that makes messages nice for Textedit
    // boxes.
    Callback.AddTexteditOutlet(generator.letter_queue, {'display':'letter_queue'},
                               outlet_number);

    Callback.AddTexteditOutlet(generator.symbol_queue, {'display': 'symbol_queue'},
                               outlet_number);

    Callback.AddTexteditOutlet(generator.letter_queue,
                               {'previous_character': 'current_letter'},
                               outlet_number);
  }
};

#endif  // __MORSE_CALLBACKS
