#ifndef __MORSE_TABLE
#define __MORSE_TABLE

// Morse.Table is a class that maps letters to Morse symbols.
Morse.Table = function() {
  var raw_table = {
    '&': '. - . . .',
    "'": '. - - - - .',
    '@': '. - - . - .',
    ':': '- - - . . .',
    ',': '- - . . - -',
    '$': '. . . - . . -',
    '=': '- . . . -',
    '!': '- . - . - -',
    '-': '- . . . . -',
    ')': '- . - - . -',
    '(': '- . - - .',
    '.': '. - . - . -',
    '+': '. - . - .',
    '?': '. . - - . .',
    '"': '. - . . - .',
    ';': '- . - . - .',
    '/': '- . . - .',
    '_': '. . - - . -',
    '0': '- - - - -',
    '1': '. - - - -',
    '2': '. . - - -',
    '3': '. . . - -',
    '4': '. . . . -',
    '5': '. . . . .',
    '6': '- . . . .',
    '7': '- - . . .',
    '8': '- - - . .',
    '9': '- - - - .',
    'A': '. -',
    'B': '- . . .',
    'C': '- . - .',
    'D': '- . .',
    'E': '.',
    'F': '. . - .',
    'G': '- - .',
    'H': '. . . .',
    'I': '. .',
    'J': '. - - -',
    'K': '- . -',
    'L': '. - . .',
    'M': '- -',
    'N': '- .',
    'O': '- - -',
    'P': '. - - .',
    'Q': '- - . -',
    'R': '. - .',
    'S': '. . .',
    'T': '-',
    'U': '. . -',
    'V': '. . . -',
    'W': '. - -',
    'X': '- . . -',
    'Y': '- . - -',
    'Z': '- - . .',
  };

  var table = {};

  for (var letter in raw_table)
    table[letter] = raw_table[letter].replace(/[^-\.]/g, '');

  // What Morse considers a wordspace.
  var whitespace_matcher = /[\s]/;

  // Code() returns the Morse code symbols for a letter if they exist; if the
  // letter is whitespace Code() returns a space; else Code() returns undefined.
  this.Code = function(letter) {
    return whitespace_matcher.test(letter) ? ' ' : table[letter.toUpperCase()];
  };

  // Measure() takes a string and measures its weight in Morse code components.
  // Returns a map from component name to the number of occurences of that
  // component.
  this.Measure = function(string) {
    var result = {};

    for (var i in string) {
      var code = this.Code(string[i]);
      if (code == ' ') {  // It's a space.
        NumberMap.AddTo(result, 'space');
        NumberMap.AddTo(result, 'gap', -1);  // We overcounted by one gap.

      } else if (code) {  // It's a valid Morse letter.
        // There's one gap for every letter.
        NumberMap.AddTo(result, 'gap');

        // There's one sep for each symbol except the last one.
        NumberMap.AddTo(result, 'sep', code.length - 1);

        // And we count the dots and dashes in a loop.
        for (var j in code)
          NumberMap.AddTo(result, (code[j] == '.') ? 'dot' : 'dash');

      } else {
        NumberMap.AddTo(result, 'unknown');
      }
    }
    return result;
  };
};

#endif  // __MORSE_TABLE
