#ifndef __COMMAND_STATE
#define __COMMAND_STATE

#include "cycle/named_level_manager.js"
#include "state/input.js"
#include "max/max_objects.js"
#include "patch/patch.js"
#include "util/defer.js"
#include "util/functional.js"
#include "util/reader.js"

function State(js) {
  var level_manager = new NamedLevelManager(
    ['Scene', 'Section', 'Song', 'Set', 'Setset']);

  var input = new Input();
  var midi = new MaxObjects(js);
  var patch = new Patch();
  var command_map = {};
  var me = this;

  this.Input = function(data) {
    return Functional.Chain(command_map, input.MapInput(data));
  };

  this.Read = function(filename) {
    return Reader.ReadData(filename, me.context);
  };

  this.SetCommandMap = function(map) {
    command_map = map;
  };

  this.context = {
    'BankPC': patch.BankPC,
    'Command': {'SetMap': this.SetCommandMap},
    'Defer': Defer,
    'Input': input.context,
    'Midi': midi.context,
    'Patch': patch,
    'Read': this.Read,
  };

  for (var name in level_manager.context)
    this.context[name] = level_manager.context[name];
};

#endif  // __COMMAND_STATE
