#ifndef __NUMBER_MAP
#define __NUMBER_MAP

// NumberMap is a namespace for function to manipulate maps from names to
// numbers: accessing them, accumulating into them and taking inner products.
var NumberMap = new Object();

// NumberMap.Get() gets an entry from a map or 0 if it doesn't exist.
NumberMap.Get = function(map, name) {
  return map[name] || 0;
};

// NumberMap.AddTo() accumulates a delta (default 1) into named map member.
NumberMap.AddTo = function(map, name, delta) {
  if (delta == undefined)
    delta = 1;
  map[name] = delta + NumberMap.Get(map, name);
  return map;
};

// NumberMap.Dot() takes the inner product of two Objects, performed over
// numeric fields only.  See http://en.wikipedia.org/wiki/Dot_product.
NumberMap.Dot = function(x, y) {
  var value = 0;
  for (var name in x) {
    if (name in y) {
      var x_val = NumberMap.Get(x, name);
      if (typeof(x_val) == 'number') {
        var y_val = NumberMap.Get(y, name);
        if (typeof(y_val) == 'number')
          value += x_val * y_val;
      }
    }
  }
  return value;
};


#endif  // __NUMBER_MAP
