#ifndef __TESTING
#define __TESTING

#include "util/print.js"

var Testing = new function() {
  var test_count;
  var fail_count;
  var name_map;
  var functions = [];

  function Reset() {
    Testing.results = [];
    Testing.verbose = false;  // true
    Testing.verbose_compare = true;

    test_count = 0;
    fail_count = 0;
    name_map = {};
  };

  function Start(name) {
    var count = (name in name_map) ? (name_map[name] + 1) : 1;
    name_map[name] = count;
    if (Testing.verbose)
      post('Test:', name, count, '\n');

    ++test_count;
  };

  function Fail(name, results, expected) {
    ++fail_count;
    post('FAIL:', name, name_map[name], '\n');
    post('Actual result:', Print(results).slice(0, 255), '\n');
    post('Expect result:',  Print(expected).slice(0, 255), '\n');
  };

  this.ExpectEqual = function(name, results, expected) {
    Start(name);
    return Testing.Compare(results, expected) || Fail(name, results, expected);
  };

  this.TestFunction = function(f) {
    functions.push(f);
  };

  this.ExpectFunction = function(name, f, target, data, expected, side_effect) {
    Start(name);
    if (side_effect)
      Testing.results = [];

    var results;
    try {
      results = f.apply(target, data);
      if (side_effect)
        results = Testing.results;

      if (Testing.Compare(results, expected))
        return true;
    } catch (e) {
      results = 'Exception ' + e.name + ': ' + e.message;
    }
    return Fail(name, results, expected);
  };

  this.Run = function() {
    post('--- start\n');
    Reset();

    for (var i = 0; i < functions.length; ++i)
      functions[i]();

    if (fail_count)
      post(fail_count, 'tests in', test_count, 'FAILED.');
    else
      post('\nAll', test_count, 'tests succeeded.');

    post('\n--- end\n');
    post('---\n');
  };
}();

#endif  // __TESTING
