#ifndef __SONG_CONTEXT
#define __SONG_CONTEXT

#include "apply.js"
#include "program_changes.js"
#include "reset_flag.js"

function SongContext() {
  this.input__map = {};

  var me = this;  // So this can't get overridden.

  this.cycle = function(type, delta) {
    if (delta == undefined)
      delta = 1;

    return (type == _SCENE) ?
      function() { next_scene(delta); } :
      function() {
        _cycle_program(type, delta);
        _reset_reset_flag();
      };
  };

  this.set_input_map = function(map) {
    return function() { me.input_map = map; }
  };

  this.program_change = function(patch) {
    return function() { _output_pc(patch[0], patch[1]); }
  };

  this.scene = function(scene, song) {
    return function() { _run_scene(scene, song); };
  };

  function add_to_map(map, value_keys) {
    var value = value_keys.shift(0);
    for (var i in value_keys) // ?
      map[value_keys[i]] = value;

    return map;
  };

  this.map = function(_) {
    return add_to_map({}, arrayfromargs(arguments));
  };

  this.edit_map = function(map, _) {
    return add_to_map(map, arrayfromargs(arguments).slice(1));
  };

  this.cond = function(condition, yes, no, depth) {
    if (!depth)
      return condition ? yes : no;

    function defer(f, x) { return function() { f(x); } }
    return function(x) {
      return this.cond(condition(x), defer(yes, x), defer(no, x), depth - 1);
    }
  };

  this.set_map_trigger = function(fn) {
    function next_wx(pc) {
      if (pc != 4)
        fn(pc);
      else
        _perhaps_reset();
    };

    return set_input_map({'pedal': fn, 'wxpc': next_wx});
  };

  this.set_scene_trigger = function(scene, song) {
    function f() { _run_scene(scene, song); };
    return set_map_trigger(f);
  };

  this.base_wx_map = {
    0: this.cycle(PATCH, 1),
    1: this.cycle(CATEGORY, 1),
    2: this.cycle(GROUP, 1),
    3: this.cycle(SCENE, 1),
    4: _perhaps_reset,
  };

  this.base_pedal_fn = function(pc) {
    _outlet(_MIDI, ['live', 'note', pc])
  };

  this.loop_message = [_outlet, [_MIDI, 'live', 'pc', 4]];
  this.base_map = { 'wxpc': _base_wx_map, 'pedal': _base_pedal_fn };
};


#endif  // __SONG_CONTEXT
